-----------------------------------------------------------------------------
--	File	 : /lua/Shield.lua
--	Author(s): Resin Smoker
--	Summary  : Base Shield Definition for Projectile Reflections
--	Copyright  2014 All rights reserved.
-----------------------------------------------------------------------------
do--(start of non-destructive hook)

	--> Triggers all logs to report if flag set true, otherwise use the local flags within each function for specific outputs
	local masterDebug = false
	--> % chance that a projectile will reflect on contact with shield	
	local chanceShieldReflection = 20
	--> Misc lua called		
	local oldUnitShield = import('/lua/shield.lua').UnitShield
	
	local oldShield = Shield
	Shield = Class( oldShield ) {
			
		OnCollisionCheck = function( self, other )
			local myDebug = true
			if myDebug and masterDebug then
				LOG('	OnCollisionCheck (/hook/lua/shield.lua), 	self ID: ', self:GetEntityId(), '	proj ID: ', other:GetEntityId(), '	Shield IsOn ', self:IsOn() )
			end										
			--> Verify that the shield and impacting entities are still valid			
			if not self:BeenDestroyed() and self:IsOn() and not other:BeenDestroyed() then
				--> Verify that impacting entity is a projectile and that there is a signifigant damage amount to be passed
				if IsProjectile(other) and other.DamageData.DamageAmount > 1 then																								
					--> Try to prevent the same projectile from impacting over and over again (1st check)
					if not other.LastImpact or other.LastImpact != self then													
						--> Check to see if this projectile has not reflected off another shield previously (2nd check)
						if not other:RtnReflected() then																
							--> Ensure that a our projectile isnt from an ally or check to see if its tagged to collide with friendlies 				
							if not IsAlly(  self:GetArmy(), other:GetArmy() ) or other.DamageData.CollideFriendly then													
								--> Ignore Nukes, torpedos and most missiles
								if not EntityCategoryContains( categories.NUKE, other ) and not EntityCategoryContains( categories.MISSILE, other ) and not EntityCategoryContains( categories.TORPEDO, other ) then								
									--> Randomly determine if the projectile will reflect then call the projectiles ValidateProjReflect to test the impactors reflectivity
									if Random( 1, 100 ) <= chanceShieldReflection and other:ValidateProjReflect( self ) then									
										if masterDebug and myDebug then
											LOG('		OnCollisionCheck(/hook/lua/shield.lua), ValidateProjReflect Returned true to be allow the projectile reflection to take place')
										end
										--> After projectile reflects and splits damage, allow normal impact event via oldShield.OnCollisionCheck 																			
									elseif masterDebug and myDebug then
										LOG('		OnCollisionCheck (/hook/lua/shield.lua), Projectile was rejected for reflection or was deemed invalid')
									end																				
								elseif masterDebug and myDebug then
									LOG('		OnCollisionCheck (/hook/lua/shield.lua), Allowing Nuke pass or missile to impact normaly')				
								end
							elseif masterDebug and myDebug then
								LOG('		OnCollisionCheck (/hook/lua/shield.lua), Projectile is most likely from an ally or its tagged not allow collisions with friendlies ','	CollideFriendly: ', other.DamageData.CollideFriendly or nil )
							end	
						else
							if masterDebug and myDebug then
								LOG('		OnCollisionCheck (/hook/lua/shield.lua), Projectile impacted a shield once before (2nd check), returning true')
							end
							return true
						end					
					else
						if masterDebug and myDebug then
							LOG('		OnCollisionCheck (/hook/lua/shield.lua), Projectile impacted this shield once before (1st check), returning false')
						end					
						return false
					end																			
				elseif masterDebug and myDebug then
					LOG('		OnCollisionCheck (/hook/lua/shield.lua), Impactor is not a damaging projectile, most likely its a projectile used for effects, running oldShield.OnCollisionCheck')							
				end							
			else
				if masterDebug and myDebug then
					LOG('		OnCollisionCheck (/hook/lua/shield.lua), "self" or "other" has been destroyed or shield is not powered up, returning false')
				end
				return false
			end
																							
	        --> Normal impact event via oldShield.OnCollisionCheck
			if masterDebug and myDebug then
				LOG('		OnCollisionCheck (/hook/lua/shield.lua), Calling oldShield.OnCollisionCheck')
			end		        							
			return oldShield.OnCollisionCheck( self, other )			
		end,
	}
			
end--(end of non-destructive hook)